@@ -144,7 +144,8 @@ AppWrapper <- setRefClass(
   'AppWrapper',
   fields = list(
     .app = 'ANY',
-    .wsconns = 'environment'
+    .wsconns = 'environment',
+    .supportsOnHeaders = 'logical'
   ),
   methods = list(
     initialize = function(app) {
@@ -152,9 +153,12 @@ AppWrapper <- setRefClass(
         .app <<- list(call=app)
       else
         .app <<- app
+      
+      # .app$onHeaders can error (e.g. if .app is a reference class)
+      .supportsOnHeaders <<- isTRUE(try(!is.null(.app$onHeaders), silent=TRUE))
     },
     onHeaders = function(req) {
-      if (is.null(.app$onHeaders))
+      if (!.supportsOnHeaders)
         return(NULL)
 
       rookCall(.app$onHeaders, req)
@@ -437,7 +441,7 @@ stopServer <- function(handle) {
 #'   listened on. Note that on most Unix-like systems including Linux and Mac OS
 #'   X, port numbers smaller than 1025 require root privileges.
 #' @param app A collection of functions that define your application. See 
-#'   Details.
+#'   \code{\link{startServer}}.
 #' @param interruptIntervalMs How often to check for interrupt. The default 
 #'   should be appropriate for most situations.
 #'   
